/* Solves a first order ODE of the form dy/dx = -x*y, with y(0) = 1,
   using Euler's method.  Solution is evaluated at x = 1. 

   Solutions are calculated for N different values of the step size,
   starting with EPS_INIT and decreasing by a factor of STEP_FAC each
   iteration.  N is given as a command-line argument to the program.
   Outputs are the stepsize and error for each case. */

#define EPS_INIT 0.01
#define STEP_FAC 2.0

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

/* ============================================================= */
/* This is appropriate for y = exp(-x^2/2). */
double f (double x, double y)
{
  return -x*y;
}

/* ============================================================= */
int main (int argc, char *argv[])
{
  double x, y, eps;
  int N, j;

  if (argc < 2) {
    printf("Error: must supply nsteps.\n");
    exit(1);
  }
  else
    N = atoi (argv[1]);

  eps = EPS_INIT;

  for (j=0; j<N; j++) {
    x   = 0.0;
    y   = 1.0;
    while (x < 1.0) {
      y += eps*f(x,y);
      x += eps;
    }
    printf("%lf\t%le\n", eps, fabs(y-exp(-0.5*x*x)));
    eps /= STEP_FAC;
  }
  return 0;
}
