/* Solves a first order ODE of the form dy/dx = -x*y, with y(0) = 1,
   using the second-order RK algorithm.  Solution is evaluated at x =
   1.

   Solutions are calculated for N different values of the step size,
   starting with EPS_INIT and decreasing by a factor of STEP_FAC each
   iteration.  N is given as a command-line argument to the program.
   Outputs are the stepsize and error for each case. */

#define EPS_INIT 0.1
#define STEP_FAC 2.0

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

/* ============================================================= */
/* This is appropriate for y = exp(-x^2/2). */
double f (double x, double y)
{
  return -x*y;
}

/* ============================================================= */
int main (int argc, char *argv[])
{
  double x, y, eps, k1, k2;
  int N, j;
  
  if (argc < 2) {
    printf("Error: must supply nsteps.\n");
    exit(1);
  }
  else
    N = atoi (argv[1]);

  eps = EPS_INIT;

  for (j=0; j<N; j++) {
    x = 0.0L;
    y = 1.0L;
    while (x < 1.0) {
      k1 = eps*f(x,y);
      k2 = eps*f(x + 0.5L*eps, y + 0.5L*k1);
      y += k2;
      x += eps;
    }
    printf("%lf\t%le\n", eps, fabs(y-expl(-0.5L*x*x)));
    eps /= STEP_FAC;
  }
  return 0;
}
