/* Solves Newton's second law for a simple harmonic oscillator with
   x(0) = 1 and v(0) = 0, using the second order RK algorithm.
   Solution is evolved for N periods, where N is specified by the user
   as a command line argument.  

   Uncomment the appropriate lines for desired output.  If data is
   recorded for each time step, output should probably be redirected
   to a file. */

#define STEPSIZE 0.01
#define OMEGA    1.0

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#ifndef PI
#define PI 4.0L*atanl(1.0)
#endif

double om2 = OMEGA*OMEGA;

/* ============================================================= */
double fv (double x)
{
  return -om2*x;
}
/* ============================================================= */
int main (int argc, char *argv[])
{
  double x, v, t, e, per, eps, k1x, k1v, k2x, k2v;
  int npers;
  
  if (argc < 2) {
    printf("Error: must supply number of periods.\n");
    exit(1);
  }
  else
    npers = atoi(argv[1]);

  eps = STEPSIZE;
  per = 2.0*PI/OMEGA;

  /* Set initial values */
  t = 0.0;
  x = 1.0;
  v = 0.0;

  /* Evolve a while... */
  while (t < npers*per) {
    k1x = eps*v;
    k1v = eps*fv(x);
    k2x = eps*(v + 0.5*k1v);
    k2v = eps*fv(x + 0.5*k1x);
    x += k2x;
    v += k2v;
    t += eps;
    printf("%lf\t%le\n", t, x);
/*     e = v*v + om2*x*x; */
/*     printf("%lf\t%le\n", t, e); */
  }
  return 0;
}
